@@ -125,9 +125,9 @@ cuda_kernels_impl = [
     "awkward_reduce_max_complex",
     "awkward_reduce_min",
     "awkward_reduce_min_complex",
-    "awkward_reduce_argmin",
+    # "awkward_reduce_argmin",
     "awkward_reduce_argmin_complex",
-    "awkward_reduce_argmax",
+    # "awkward_reduce_argmax",
     "awkward_reduce_argmax_complex",
     "awkward_reduce_sum",
     "awkward_reduce_sum_bool",
