@@ -11,11 +11,16 @@ import awkward._lookup
 
 ROOT = pytest.importorskip("ROOT")
 
-ROOT.ROOT.EnableImplicitMT(1)
-
 compiler = ROOT.gInterpreter.Declare
 
 
+@pytest.fixture(scope="module", autouse=True)
+def manage_imt():
+    ROOT.ROOT.EnableImplicitMT(1)
+    yield
+    ROOT.ROOT.DisableImplicitMT()
+
+
 def test_unknown_column_type():
     example1 = ak.Array([1.1, 2.2, 3.3, 4.4, 5.5])
     data_frame = ak.to_rdataframe(
