@@ -3,6 +3,7 @@
 from __future__ import annotations
 
 import numpy as np
+import pytest
 
 import awkward as ak
 
@@ -14,7 +15,8 @@ def test_many():
     assert result.tolist() == [{"a": 3}, {"c": 3}, {"d": 3}, {"e": 3}]
 
 
-def test_validity_error_simple():
+@pytest.mark.parametrize("forget_length", [False, True])
+def test_validity_error_simple(forget_length):
     layout = ak.contents.UnionArray(
         ak.index.Index8(np.array([0, 0, 0, 0, 1, 1, 1, 1, 1], dtype=np.int8)),
         ak.index.Index64(np.array([0, 1, 2, 3, 0, 1, 2, 3, 4], dtype=np.int64)),
@@ -22,9 +24,13 @@ def test_validity_error_simple():
     )
     assert layout.to_list() == [1, 2, 3, 4, 5, 6, 7, 8, 9]
     assert "content(1) is mergeable with content(0)" in ak.validity_error(layout)
+    assert "content(1) is mergeable with content(0)" in ak.validity_error(
+        layout.to_typetracer(forget_length)
+    )
 
 
-def test_validity_error_complex():
+@pytest.mark.parametrize("forget_length", [False, True])
+def test_validity_error_complex(forget_length):
     layout = ak.contents.UnionArray(
         ak.index.Index8(np.array([0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2], dtype=np.int8)),
         ak.index.Index64(np.array([0, 1, 2, 3, 0, 1, 2, 3, 0, 1, 2], dtype=np.int64)),
@@ -36,3 +42,6 @@ def test_validity_error_complex():
     )
     assert layout.to_list() == [1, 2, 3, 4, "a", "b", "c", "d", 5, 6, 7]
     assert "content(2) is mergeable with content(0)" in ak.validity_error(layout)
+    assert "content(2) is mergeable with content(0)" in ak.validity_error(
+        layout.to_typetracer(forget_length)
+    )
