@@ -11,7 +11,7 @@ import awkward as ak
 def test():
     arr = ak.Array([{"x": 1}, {"x": 2}])
     # Fill with
-    result = ak.full_like(arr, np.datetime64(20, "s"), dtype="<M8[s]")
+    result = ak.full_like(arr, np.datetime64(20, "s"), dtype="M8[s]")
     assert result.layout.is_equal_to(
         ak.contents.RecordArray(
             [
@@ -27,7 +27,7 @@ def test():
 def test_typetracer():
     arr = ak.Array([{"x": 1}, {"x": 2}], backend="typetracer")
     # Fill with
-    result = ak.full_like(arr, np.datetime64(20, "s"), dtype="<M8[s]")
+    result = ak.full_like(arr, np.datetime64(20, "s"), dtype="M8[s]")
     assert result.layout.form == (
         ak.forms.RecordForm(
             [ak.forms.NumpyForm("datetime64[s]")],
